@@ -637,71 +637,67 @@ pack_object(PyObject *object) {
   // had to add this for basic 64 and unsigned data to get packed right .. Not
   // sure if we can just do the rest this way..
 
- switch(pack_type)
-  {
+  switch (pack_type) {
   case PT_int64:
-      if(PyLong_Check(object))
-      {
-            pack_int64(PyLong_AsLongLong(object));
-            return;
-      }
+    if (PyLong_Check(object)) {
+      pack_int64(PyLong_AsLongLong(object));
+      return;
+    }
 #if PY_MAJOR_VERSION < 3
-      else if (PyInt_Check(object))
-      {
-            pack_int64(PyInt_AsLong(object));
-            return;
-      }
+    else if (PyInt_Check(object)) {
+      pack_int64(PyInt_AsLong(object));
+      return;
+    }
 #endif
-      break;
+    break;
+
   case PT_uint64:
-      if(PyLong_Check(object))
-      {
-            pack_uint64(PyLong_AsUnsignedLongLong(object));
-            return;
-      }
+    if (PyLong_Check(object)) {
+      pack_uint64(PyLong_AsUnsignedLongLong(object));
+      return;
+    }
 #if PY_MAJOR_VERSION < 3
-      else if(PyInt_Check(object))
-      {
-            PyObject  *obj1 = PyNumber_Long(object);
-            pack_int(PyLong_AsUnsignedLongLong(obj1));
-            Py_DECREF(obj1);
-            return;
-      }
+    else if (PyInt_Check(object)) {
+      PyObject  *obj1 = PyNumber_Long(object);
+      pack_int(PyLong_AsUnsignedLongLong(obj1));
+      Py_DECREF(obj1);
+      return;
+    }
 #endif
-      break;
+    break;
+
   case PT_int:
-      if(PyLong_Check(object))
-      {
-            pack_int(PyLong_AsLong(object));
-            return;
-      }
+    if (PyLong_Check(object)) {
+      pack_int(PyLong_AsLong(object));
+      return;
+    }
 #if PY_MAJOR_VERSION < 3
-      else if (PyInt_Check(object))
-      {
-            pack_int(PyInt_AsLong(object));
-            return;
-      }
+    else if (PyInt_Check(object)) {
+      pack_int(PyInt_AsLong(object));
+      return;
+    }
 #endif
-      break;
+    break;
+
   case PT_uint:
-      if(PyLong_Check(object))
-      {
-            pack_uint(PyLong_AsUnsignedLong(object));
-            return;
-      }
+    if (PyLong_Check(object)) {
+      pack_uint(PyLong_AsUnsignedLong(object));
+      return;
+    }
 #if PY_MAJOR_VERSION < 3
-      else if (PyInt_Check(object))
-      {
-            PyObject *obj1 = PyNumber_Long(object);
-            pack_uint(PyLong_AsUnsignedLong(obj1));
-            Py_DECREF(obj1);
-            return;
-      }
+    else if (PyInt_Check(object)) {
+      PyObject *obj1 = PyNumber_Long(object);
+      pack_uint(PyLong_AsUnsignedLong(obj1));
+      Py_DECREF(obj1);
+      return;
+    }
 #endif
-      break;
+    break;
+
   default:
-      break;
+    break;
   }
+
   if (PyLong_Check(object)) {
     pack_int(PyLong_AsLong(object));
 #if PY_MAJOR_VERSION < 3
@@ -851,7 +847,7 @@ unpack_object() {
     {
       unsigned int value = unpack_uint();
 #if PY_MAJOR_VERSION >= 3
-      object = PyLong_FromLong(value);
+      object = PyLong_FromUnsignedLong(value);
 #else
       if (value & 0x80000000) {
         object = PyLong_FromUnsignedLong(value);
@@ -893,6 +889,10 @@ unpack_object() {
       unpack_string(str);
 #if PY_MAJOR_VERSION >= 3
       object = PyUnicode_FromStringAndSize(str.data(), str.size());
+      if (object == nullptr) {
+        nassert_raise("Unable to decode UTF-8 string; use blob type for binary data");
+        return nullptr;
+      }
 #else
       object = PyString_FromStringAndSize(str.data(), str.size());
 #endif
@@ -1102,7 +1102,7 @@ enquote_string(ostream &out, char quote_mark, const string &str) {
     if ((*pi) == quote_mark || (*pi) == '\\') {
       out << '\\' << (*pi);
 
-    } else if (!isprint(*pi)) {
+    } else if (!isprint(*pi) || (*pi) == '\t') {
       char buffer[10];
       sprintf(buffer, "%02x", (unsigned char)(*pi));
       out << "\\x" << buffer;
