@@ -1,6 +1,11 @@
 """ This module is used to build a graphical installer
 or a standalone executable from a p3d file. It will try
-to build for as many platforms as possible. """
+to build for as many platforms as possible.
+
+.. deprecated:: 1.10.0
+   The p3d packaging system has been replaced with the new setuptools-based
+   system.  See the :ref:`distribution` manual section.
+"""
 
 __all__ = ["Standalone", "Installer"]
 
@@ -8,9 +13,10 @@ import os, sys, subprocess, tarfile, shutil, time, zipfile, socket, getpass, str
 import gzip, plistlib
 from direct.directnotify.DirectNotifyGlobal import *
 from direct.showbase.AppRunnerGlobal import appRunner
-from panda3d.core import PandaSystem, HTTPClient, Filename, VirtualFileSystem, Multifile
+from panda3d.core import PandaSystem, Filename, VirtualFileSystem, Multifile
 from panda3d.core import TiXmlDocument, TiXmlDeclaration, TiXmlElement, readXmlStream
 from panda3d.core import PNMImage, PNMFileTypeRegistry, StringStream
+from panda3d import core
 from direct.stdpy.file import *
 from direct.p3d.HostInfo import HostInfo
 # This is important for some reason
@@ -82,7 +88,7 @@ class Standalone:
         self.tempDir.makeDir()
         self.host = HostInfo(PandaSystem.getPackageHostUrl(), appRunner = appRunner, hostDir = self.tempDir, asMirror = False, perPlatform = True)
 
-        self.http = HTTPClient.getGlobalPtr()
+        self.http = core.HTTPClient.getGlobalPtr()
         if not self.host.hasContentsFile:
             if not self.host.readContentsFile():
                 if not self.host.downloadContentsFile(self.http):
@@ -240,7 +246,7 @@ class PackageTree:
         self.hostUrl = hostUrl
         self.hostDir = Filename(hostDir)
         self.hostDir.makeDir()
-        self.http = HTTPClient.getGlobalPtr()
+        self.http = core.HTTPClient.getGlobalPtr()
 
     def getHost(self, hostUrl):
         if hostUrl in self.hosts:
@@ -344,6 +350,7 @@ class Icon:
             if required_size * 2 in sizes:
                 from_size = required_size * 2
             else:
+                from_size = 0
                 for from_size in sizes:
                     if from_size > required_size:
                         break
@@ -352,8 +359,7 @@ class Icon:
                 Icon.notify.warning("Generating %dx%d icon by scaling down %dx%d image" % (required_size, required_size, from_size, from_size))
 
                 image = PNMImage(required_size, required_size)
-                if self.images[from_size].hasAlpha():
-                    image.addAlpha()
+                image.setColorType(self.images[from_size].getColorType())
                 image.quickFilterFrom(self.images[from_size])
                 self.images[required_size] = image
             else:
@@ -367,7 +373,7 @@ class Icon:
         # XOR mask
         if bpp == 24:
             # Align rows to 4-byte boundary
-            rowalign = '\0' * (-(size * 3) & 3)
+            rowalign = b'\0' * (-(size * 3) & 3)
             for y in xrange(size):
                 for x in xrange(size):
                     r, g, b = image.getXel(x, size - y - 1)
@@ -383,35 +389,14 @@ class Icon:
         elif bpp == 8:
             # We'll have to generate a palette of 256 colors.
             hist = PNMImage.Histogram()
-            if image.hasAlpha():
-                # Make a copy without alpha channel.
-                image2 = PNMImage(image)
+            image2 = PNMImage(image)
+            if image2.hasAlpha():
                 image2.premultiplyAlpha()
                 image2.removeAlpha()
-            else:
-                image2 = image
+            image2.quantize(256)
             image2.make_histogram(hist)
             colors = list(hist.get_pixels())
-            if len(colors) > 256:
-                # Palette too large; remove infrequent colors.
-                colors.sort(key=hist.get_count, reverse=True)
-
-                # Find the closest color on the palette matching each color
-                # that didn't fit.  This is certainly not the best palette
-                # generation code, but it'll do for now.
-                closest_indices = []
-                for color in colors[256:]:
-                    closest_index = 0
-                    closest_diff = 1025
-                    for i, closest_color in enumerate(colors[:256]):
-                        diff = abs(color.get_red() - closest_color.get_red()) \
-                             + abs(color.get_green() - closest_color.get_green()) \
-                             + abs(color.get_blue() - closest_color.get_blue())
-                        if diff < closest_diff:
-                            closest_index = i
-                            closest_diff = diff
-                    assert closest_diff < 100
-                    closest_indices.append(closest_index)
+            assert len(colors) <= 256
 
             # Write the palette.
             i = 0
@@ -429,9 +414,6 @@ class Icon:
                 for x in xrange(size):
                     pixel = image2.get_pixel(x, size - y - 1)
                     index = colors.index(pixel)
-                    if index >= 256:
-                        # Find closest pixel instead.
-                        index = closest_indices[index - 256]
                     fp.write(struct.pack('<B', index))
                 fp.write(rowalign)
         else:
@@ -503,7 +485,7 @@ class Icon:
             if size > 256:
                 continue
             elif size == 256:
-                ico.write('\0\0')
+                ico.write(b'\0\0')
             else:
                 ico.write(struct.pack('<BB', size, size))
 
