@@ -1,4 +1,6 @@
 from . import ForceGroup
+from direct.showbase.PhysicsManagerGlobal import physicsMgr
+
 
 class GlobalForceGroup(ForceGroup.ForceGroup):
 
@@ -6,7 +8,7 @@ class GlobalForceGroup(ForceGroup.ForceGroup):
         ForceGroup.ForceGroup.__init__(self, name)
 
     def addForce(self, force):
-        ForceGroup.ForceGroup.addForce(force)
+        ForceGroup.ForceGroup.addForce(self, force)
         if (force.isLinear() == 0):
             # Physics manager will need an angular integrator
             base.addAngularIntegrator()
@@ -16,7 +18,7 @@ class GlobalForceGroup(ForceGroup.ForceGroup):
             physicsMgr.addAngularForce(force)
 
     def removeForce(self, force):
-        ForceGroup.ForceGroup.removeForce(force)
+        ForceGroup.ForceGroup.removeForce(self, force)
         if (force.isLinear() == 1):
             physicsMgr.removeLinearForce(force)
         else:
