@@ -248,7 +248,10 @@ clear_colliders() {
 }
 
 /**
- *
+ * Perform the traversal. Begins at the indicated root and detects all
+ * collisions with any of its collider objects against nodes at or below the
+ * indicated root, calling the appropriate CollisionHandler for each detected
+ * collision.
  */
 void CollisionTraverser::
 traverse(const NodePath &root) {
@@ -351,7 +354,7 @@ traverse(const NodePath &root) {
   CollisionBox::flush_level();
 }
 
-#ifdef DO_COLLISION_RECORDING
+#if defined(DO_COLLISION_RECORDING) || !defined(CPPPARSER)
 /**
  * Uses the indicated CollisionRecorder object to start recording the
  * intersection tests made by each subsequent call to traverse() on this
@@ -370,6 +373,7 @@ traverse(const NodePath &root) {
  */
 void CollisionTraverser::
 set_recorder(CollisionRecorder *recorder) {
+#ifdef DO_COLLISION_RECORDING
   if (recorder != _recorder) {
     // Remove the old recorder, if any.
     if (_recorder != nullptr) {
@@ -389,6 +393,7 @@ set_recorder(CollisionRecorder *recorder) {
       _recorder->_trav = this;
     }
   }
+#endif
 }
 
 /**
@@ -399,11 +404,15 @@ set_recorder(CollisionRecorder *recorder) {
  */
 CollisionVisualizer *CollisionTraverser::
 show_collisions(const NodePath &root) {
+#ifdef DO_COLLISION_RECORDING
   hide_collisions();
   CollisionVisualizer *viz = new CollisionVisualizer("show_collisions");
   _collision_visualizer_np = root.attach_new_node(viz);
   set_recorder(viz);
   return viz;
+#else
+  return nullptr;
+#endif
 }
 
 /**
@@ -411,10 +420,12 @@ show_collisions(const NodePath &root) {
  */
 void CollisionTraverser::
 hide_collisions() {
+#ifdef DO_COLLISION_RECORDING
   if (!_collision_visualizer_np.is_empty()) {
     _collision_visualizer_np.remove_node();
   }
   clear_recorder();
+#endif
 }
 
 #endif  // DO_COLLISION_RECORDING
