@@ -55,13 +55,24 @@
 
 #if defined(WIN32)
   #define WINDOWS_LEAN_AND_MEAN
-  #include <WinSock2.h>
+  #include <winsock2.h>
   #include <wtypes.h>
   #undef WINDOWS_LEAN_AND_MEAN
 #else
   #include <sys/time.h>
 #endif
 
+#if defined(__APPLE__) && !defined(CPPPARSER)
+#include <AvailabilityMacros.h>
+
+#if __MAC_OS_X_VERSION_MIN_REQUIRED >= 1070
+extern "C" {
+  void *objc_autoreleasePoolPush();
+  void objc_autoreleasePoolPop(void *);
+};
+#endif
+#endif
+
 using std::string;
 
 PT(GraphicsEngine) GraphicsEngine::_global_ptr;
@@ -191,7 +202,7 @@ GraphicsEngine::
 
 /**
  * Specifies how future objects created via make_gsg(), make_buffer(), and
- * make_window() will be threaded.  This does not affect any already-created
+ * make_output() will be threaded.  This does not affect any already-created
  * objects.
  */
 void GraphicsEngine::
@@ -1433,6 +1444,10 @@ cull_and_draw_together(GraphicsEngine::Windows wlist,
     GraphicsOutput *win = wlist[wi];
     if (win->is_active() && win->get_gsg()->is_active()) {
       if (win->flip_ready()) {
+        if (display_cat.is_spam()) {
+          display_cat.spam()
+            << "Flipping window " << win->get_name() << "\n";
+        }
         {
           PStatTimer timer(GraphicsEngine::_flip_begin_pcollector, current_thread);
           win->begin_flip();
@@ -1452,6 +1467,11 @@ cull_and_draw_together(GraphicsEngine::Windows wlist,
           gsg->pop_group_marker();
         }
 
+        if (display_cat.is_spam()) {
+          display_cat.spam()
+            << "Culling and drawing window " << win->get_name() << "\n";
+        }
+
         int num_display_regions = win->get_num_active_display_regions();
         for (int i = 0; i < num_display_regions; i++) {
           PT(DisplayRegion) dr = win->get_active_display_region(i);
@@ -1463,6 +1483,10 @@ cull_and_draw_together(GraphicsEngine::Windows wlist,
 
         if (_auto_flip) {
           if (win->flip_ready()) {
+            if (display_cat.is_spam()) {
+              display_cat.spam()
+                << "Flipping window " << win->get_name() << "\n";
+            }
             {
               PStatTimer timer(GraphicsEngine::_flip_begin_pcollector, current_thread);
               win->begin_flip();
@@ -1559,6 +1583,11 @@ cull_to_bins(GraphicsEngine::Windows wlist, Thread *current_thread) {
   for (size_t wi = 0; wi < wlist_size; ++wi) {
     GraphicsOutput *win = wlist[wi];
     if (win->is_active() && win->get_gsg()->is_active()) {
+      if (display_cat.is_spam()) {
+        display_cat.spam()
+          << "Culling window " << win->get_name() << "\n";
+      }
+
       GraphicsStateGuardian *gsg = win->get_gsg();
       PStatTimer timer(win->get_cull_window_pcollector(), current_thread);
       int num_display_regions = win->get_num_active_display_regions();
@@ -1658,6 +1687,11 @@ draw_bins(const GraphicsEngine::Windows &wlist, Thread *current_thread) {
 
       GraphicsOutput *host = win->get_host();
       if (host->flip_ready()) {
+        if (display_cat.is_spam()) {
+          display_cat.spam()
+            << "Flipping window " << host->get_name()
+            << " before drawing window " << win->get_name() << "\n";
+        }
         {
           // We can't use a PStatGPUTimer before begin_frame, so when using
           // GPU timing, it is advisable to set auto-flip to #t.
@@ -1705,6 +1739,10 @@ draw_bins(const GraphicsEngine::Windows &wlist, Thread *current_thread) {
 #endif
 
           if (win->flip_ready()) {
+            if (display_cat.is_spam()) {
+              display_cat.spam()
+                << "Flipping window " << win->get_name() << "\n";
+            }
             {
               // begin_flip doesn't do anything interesting, let's not waste
               // two timer queries on that.
@@ -1780,6 +1818,11 @@ flip_windows(const GraphicsEngine::Windows &wlist, Thread *current_thread) {
   for (i = 0; i < num_windows; ++i) {
     GraphicsOutput *win = wlist[i];
     if (win->flip_ready()) {
+      if (display_cat.is_spam()) {
+        display_cat.spam()
+          << "Flipping window " << win->get_name() << "\n";
+      }
+
       nassertv(warray_count < num_windows);
       warray[warray_count] = win;
       ++warray_count;
@@ -2095,7 +2138,7 @@ do_draw(GraphicsOutput *win, GraphicsStateGuardian *gsg, DisplayRegion *dr, Thre
 }
 
 /**
- * An internal function called by make_window() and make_buffer() and similar
+ * An internal function called by make_output() and make_buffer() and similar
  * functions to add the newly-created GraphicsOutput object to the engine's
  * list of windows, and to request that the window be opened.
  */
@@ -2186,6 +2229,11 @@ void GraphicsEngine::
 do_resort_windows() {
   _windows_sorted = true;
 
+  if (display_cat.is_spam()) {
+    display_cat.spam()
+      << "Re-sorting window list.\n";
+  }
+
   _app.resort_windows();
   Threads::const_iterator ti;
   for (ti = _threads.begin(); ti != _threads.end(); ++ti) {
@@ -2513,11 +2561,20 @@ do_frame(GraphicsEngine *engine, Thread *current_thread) {
   PStatTimer timer(engine->_do_frame_pcollector, current_thread);
   LightReMutexHolder holder(_wl_lock);
 
+#if defined(__APPLE__) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 1070
+  // Enclose the entire frame in an autorelease pool.
+  void *pool = objc_autoreleasePoolPush();
+#endif
+
   engine->cull_to_bins(_cull, current_thread);
   engine->cull_and_draw_together(_cdraw, current_thread);
   engine->draw_bins(_draw, current_thread);
   engine->process_events(_window, current_thread);
 
+#if defined(__APPLE__) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 1070
+  objc_autoreleasePoolPop(pool);
+#endif
+
   // If any GSG's on the list have no more outstanding pointers, clean them
   // up.  (We are in the draw thread for all of these GSG's.)
   if (any_done_gsgs()) {
@@ -2549,10 +2606,18 @@ void GraphicsEngine::WindowRenderer::
 do_windows(GraphicsEngine *engine, Thread *current_thread) {
   LightReMutexHolder holder(_wl_lock);
 
+#if defined(__APPLE__) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 1070
+  void *pool = objc_autoreleasePoolPush();
+#endif
+
   engine->process_events(_window, current_thread);
 
   engine->make_contexts(_cdraw, current_thread);
   engine->make_contexts(_draw, current_thread);
+
+#if defined(__APPLE__) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 1070
+  objc_autoreleasePoolPop(pool);
+#endif
 }
 
 /**
