@@ -245,6 +245,7 @@ issue_parameters(GSG *gsg, int altered) {
           HRESULT hr;
           PN_stdfloat v [4];
           LMatrix4f temp_matrix = LCAST(float, *val);
+          LMatrix3f temp_matrix3;
 
           hr = D3D_OK;
 
@@ -306,6 +307,33 @@ issue_parameters(GSG *gsg, int altered) {
             hr = cgD3D9SetUniform(p, v);
             break;
 
+          case Shader::SMP_upper3x3:
+            // TRANSPOSE REQUIRED
+            temp_matrix3 = temp_matrix.get_upper_3();
+            temp_matrix3.transpose_in_place();
+            data = temp_matrix3.get_data();
+
+            hr = cgD3D9SetUniform(p, data);
+            break;
+
+          case Shader::SMP_transpose3x3:
+            // NO TRANSPOSE REQUIRED
+            temp_matrix3 = temp_matrix.get_upper_3();
+            data = temp_matrix3.get_data();
+
+            hr = cgD3D9SetUniform(p, data);
+            break;
+
+          case Shader::SMP_cell15:
+            hr = cgD3D9SetUniform(p, data + 15);
+            continue;
+          case Shader::SMP_cell14:
+            hr = cgD3D9SetUniform(p, data + 14);
+            continue;
+          case Shader::SMP_cell13:
+            hr = cgD3D9SetUniform(p, data + 13);
+            continue;
+
           default:
             dxgsg9_cat.error()
               << "issue_parameters () SMP parameter type not implemented " << spec._piece << "\n";
