@@ -862,7 +862,7 @@ void Init_D3DFORMAT_map() {
   INSERT_ELEM(D24S8);
   INSERT_ELEM(D32);
   INSERT_ELEM(INTZ);
-// NOT IN DX9 INSERT_ELEM(W11V11U10);
+  INSERT_ELEM(R32F);
   INSERT_ELEM(A2W10V10U10);
   INSERT_ELEM(ATI1);
   INSERT_ELEM(ATI2);
@@ -923,7 +923,11 @@ const char *D3DFormatStr(D3DFORMAT fmt) {
     CASESTR(D3DFMT_VERTEXDATA);
     CASESTR(D3DFMT_INDEX16);
     CASESTR(D3DFMT_INDEX32);
+    CASESTR(D3DFMT_R16F);
+    CASESTR(D3DFMT_G16R16F);
     CASESTR(D3DFMT_A16B16G16R16F);
+    CASESTR(D3DFMT_R32F);
+    CASESTR(D3DFMT_G32R32F);
     CASESTR(D3DFMT_A32B32G32R32F);
   }
 
