@@ -564,6 +564,11 @@ affects_polygon_alpha() const {
   case ET_selector:
     return true;
 
+  case ET___occlusion:
+  case ET___occlusion_metallic_roughness:
+  case ET___metallic_roughness:
+    return false;
+
   case ET_unspecified:
     break;
   }
@@ -886,6 +891,15 @@ string_env_type(const string &string) {
   } else if (cmp_nocase_uh(string, "emission") == 0) {
     return ET_emission;
 
+  } else if (cmp_nocase_uh(string, "occlusion") == 0) {
+    return ET___occlusion;
+
+  } else if (cmp_nocase_uh(string, "occlusion_metallic_roughness") == 0) {
+    return ET___occlusion_metallic_roughness;
+
+  } else if (cmp_nocase_uh(string, "metallic_roughness") == 0) {
+    return ET___metallic_roughness;
+
   } else {
     return ET_unspecified;
   }
@@ -1319,6 +1333,15 @@ ostream &operator << (ostream &out, EggTexture::EnvType type) {
 
   case EggTexture::ET_emission:
     return out << "emission";
+
+  case EggTexture::ET___occlusion:
+    return out << "occlusion";
+
+  case EggTexture::ET___occlusion_metallic_roughness:
+    return out << "occlusion_metallic_roughness";
+
+  case EggTexture::ET___metallic_roughness:
+    return out << "metallic_roughness";
   }
 
   nassertr(false, out);
