@@ -53,15 +53,18 @@ PythonTask(PyObject *func_or_coro, const std::string &name) :
     _function = func_or_coro;
     Py_INCREF(_function);
 #if PY_VERSION_HEX >= 0x03050000
-  } else if (PyCoro_CheckExact(func_or_coro)) {
-    // We also allow passing in a coroutine, because why not.
+  } else if (PyCoro_CheckExact(func_or_coro) ||
+             PyGen_CheckExact(func_or_coro) ||
+             PyType_IsSubtype(Py_TYPE(func_or_coro), &PyCoro_Type)) {
+    // We also allow passing in a coroutine.
     _generator = func_or_coro;
     Py_INCREF(_generator);
-#endif
+#else
   } else if (PyGen_CheckExact(func_or_coro)) {
     // Something emulating a coroutine.
     _generator = func_or_coro;
     Py_INCREF(_generator);
+#endif
   } else {
     nassert_raise("Invalid function passed to PythonTask");
   }
@@ -368,6 +371,17 @@ __getattr__(PyObject *attr) const {
   // tp_getattro slot (a la __getattribute__). So, we won't get here when the
   // attribute has already been found via other methods.
 
+#if PY_VERSION_HEX >= 0x030D00A1  // 3.13
+  PyObject *item = nullptr;
+  if (PyDict_GetItemRef(__dict__, attr, &item) == 0) {
+    // PyDict_GetItemRef does not raise an exception on missing attribute.
+    PyErr_Format(PyExc_AttributeError,
+                 "'PythonTask' object has no attribute '%U'",
+                 attr);
+  }
+  return item;
+
+#else  // 3.12 and lower
   PyObject *item = PyDict_GetItem(__dict__, attr);
 
   if (item == nullptr) {
@@ -387,6 +401,7 @@ __getattr__(PyObject *attr) const {
   // PyDict_GetItem returns a borrowed reference.
   Py_INCREF(item);
   return item;
+#endif
 }
 
 /**
@@ -516,7 +531,7 @@ do_python_task() {
         Py_DECREF(str);
         Py_DECREF(str2);
       }
-      if (PyCoro_CheckExact(result)) {
+      if (PyObject_TypeCheck(result, &PyCoro_Type)) {
         // If a coroutine, am_await is possible but senseless, since we can
         // just call send(None) on the coroutine itself.
         _generator = result;
@@ -534,9 +549,15 @@ do_python_task() {
     // We are calling a generator.  Use "send" rather than PyIter_Next since
     // we need to be able to read the value from a StopIteration exception.
     PyObject *func = PyObject_GetAttrString(_generator, "send");
-    nassertr(func != nullptr, DS_interrupt);
-    result = PyObject_CallFunctionObjArgs(func, Py_None, nullptr);
-    Py_DECREF(func);
+    if (func != nullptr) {
+      result = PyObject_CallOneArg(func, Py_None);
+      Py_DECREF(func);
+    } else {
+      // It has no send(), just call next() directly.
+      nassertr(Py_TYPE(_generator)->tp_iternext != nullptr, DS_interrupt);
+      PyErr_Clear();
+      result = Py_TYPE(_generator)->tp_iternext(_generator);
+    }
 
     if (result == nullptr) {
       // An error happened.  If StopIteration, that indicates the task has
@@ -606,7 +627,7 @@ do_python_task() {
       }
 
 #if PY_VERSION_HEX >= 0x03050000
-    } else if (result == Py_None && PyCoro_CheckExact(_generator)) {
+    } else if (result == Py_None && PyObject_TypeCheck(_generator, &PyCoro_Type)) {
       // Bare yield from a coroutine means to try again next frame.
       Py_DECREF(result);
       return DS_cont;
