@@ -1371,6 +1371,8 @@ release_all_shader_buffers() {
        ++bci) {
 
     BufferContext *bc = (BufferContext *)(*bci);
+    ((ShaderBuffer *)bc->_object)->clear_prepared(this);
+    bc->_object = nullptr;
     _released_shader_buffers.insert(bc);
   }
 
@@ -1656,6 +1658,12 @@ begin_frame(GraphicsStateGuardianBase *gsg, Thread *current_thread) {
   }
 
   _enqueued_index_buffers.clear();
+
+  for (ShaderBuffer *buffer : _enqueued_shader_buffers) {
+    buffer->prepare_now(this, gsg);
+  }
+
+  _enqueued_shader_buffers.clear();
 }
 
 /**
