@@ -214,6 +214,12 @@ public:
     // Color of an M_blend texture stage.
     SMO_texcolor_i,
 
+    // Constant value of the TexGenAttrib of stage i.
+    SMO_texconst_i,
+
+    // Point parameters
+    SMO_attr_pointparams,
+
     SMO_INVALID
   };
 
@@ -225,6 +231,15 @@ public:
 
     STO_stage_i,
     STO_light_i_shadow_map,
+
+    STO_ff_stage_i,
+    STO_stage_modulate_i,
+    STO_stage_add_i,
+    STO_stage_normal_i,
+    STO_stage_height_i,
+    STO_stage_selector_i,
+    STO_stage_gloss_i,
+    STO_stage_emission_i,
   };
 
   enum ShaderArgClass {
@@ -265,6 +280,7 @@ public:
     SAT_sampler_cube,
     SAT_sampler_buffer,
     SAT_sampler_cube_array,
+    SAT_sampler1d_array,
     SAT_unknown
 };
 
@@ -312,6 +328,8 @@ public:
     SSD_projection    = 0x800,
     SSD_texture      = 0x1000,
     SSD_view_transform= 0x2000,
+    SSD_tex_gen      = 0x4000,
+    SSD_render_mode  = 0x8000,
   };
 
   enum ShaderBug {
