@@ -46,6 +46,13 @@ PyObject *Extension<PandaNode>::
 __deepcopy__(PyObject *self, PyObject *memo) const {
   extern struct Dtool_PyTypedObject Dtool_PandaNode;
 
+#if PY_VERSION_HEX >= 0x030D00A1  // 3.13
+  PyObject *dupe;
+  if (PyDict_GetItemRef(memo, self, &dupe) != 0) {
+    // Already in the memo dictionary (or an error happened).
+    return dupe;
+  }
+#else
   // Borrowed reference.
   PyObject *dupe = PyDict_GetItem(memo, self);
   if (dupe != nullptr) {
@@ -53,6 +60,7 @@ __deepcopy__(PyObject *self, PyObject *memo) const {
     Py_INCREF(dupe);
     return dupe;
   }
+#endif
 
   PT(PandaNode) node_dupe = _this->copy_subgraph();
 
