@@ -17,7 +17,7 @@
 #include "pandabase.h"
 #include "selectThreadImpl.h"
 
-#if defined(WIN32_VC)
+#ifdef _WIN32
 
 #include "mutexWin32Impl.h"
 #include "pnotify.h"
@@ -50,6 +50,6 @@ private:
 
 #include "conditionVarWin32Impl.I"
 
-#endif  // WIN32_VC
+#endif  // _WIN32
 
 #endif
