@@ -453,8 +453,8 @@ png_read_data(png_structp png_ptr, png_bytep data, png_size_t length) {
  * This is our own warning handler.  It is called by the png library to issue
  * a warning message.
  */
-void PNMFileTypePNG::Reader::
-png_warning(png_structp, png_const_charp warning_msg) {
+void (PNMFileTypePNG::Reader::
+png_warning)(png_structp, png_const_charp warning_msg) {
   pnmimage_png_cat.warning()
     << warning_msg << "\n";
 }
@@ -463,8 +463,8 @@ png_warning(png_structp, png_const_charp warning_msg) {
  * This is our own error handler.  It is called by the png library to issue a
  * fatal error message.
  */
-void PNMFileTypePNG::Reader::
-png_error(png_structp png_ptr, png_const_charp error_msg) {
+void (PNMFileTypePNG::Reader::
+png_error)(png_structp png_ptr, png_const_charp error_msg) {
   pnmimage_png_cat.error()
     << error_msg << "\n";
 
@@ -923,8 +923,8 @@ png_flush_data(png_structp png_ptr) {
  * This is our own warning handler.  It is called by the png library to issue
  * a warning message.
  */
-void PNMFileTypePNG::Writer::
-png_warning(png_structp, png_const_charp warning_msg) {
+void (PNMFileTypePNG::Writer::
+png_warning)(png_structp, png_const_charp warning_msg) {
   pnmimage_png_cat.warning()
     << warning_msg << "\n";
 }
@@ -933,8 +933,8 @@ png_warning(png_structp, png_const_charp warning_msg) {
  * This is our own error handler.  It is called by the png library to issue a
  * fatal error message.
  */
-void PNMFileTypePNG::Writer::
-png_error(png_structp png_ptr, png_const_charp error_msg) {
+void (PNMFileTypePNG::Writer::
+png_error)(png_structp png_ptr, png_const_charp error_msg) {
   pnmimage_png_cat.error()
     << error_msg << "\n";
 
