@@ -173,7 +173,7 @@ private:
   bool _ime_open;
   bool _ime_active;
   bool _tracking_mouse_leaving;
-  bool _bCursor_in_WindowClientArea;
+  bool _in_adjust_z_order = false;
   HANDLE _input_device_handle[32];
   HCURSOR _cursor;
   DEVMODE _fullscreen_display_mode;
